use axum::{http::StatusCode, response::IntoResponse, Json};

use crate::{responses::registration::RegistrationResponse, types::error_code::ErrorCode};

use super::ErrorResponse;

#[derive(Debug, thiserror::Error)]
pub enum RegistrationError {
    #[error("The homeserver requires additional authentication information")]
    AdditionalAuthenticationInformation,
    #[error("UserId is missing")]
    MissingUserId,
    #[error("The desired user ID is not a valid user name")]
    InvalidUserId,
    #[error("The desired user ID is already taken")]
    UserIdTaken,
}

impl IntoResponse for RegistrationError {
    fn into_response(self) -> axum::response::Response {
        match self {
            RegistrationError::AdditionalAuthenticationInformation => (
                StatusCode::UNAUTHORIZED,
                Json(RegistrationResponse::user_interactive_authorization_info()),
            )
                .into_response(),
            RegistrationError::InvalidUserId | RegistrationError::MissingUserId => (
                StatusCode::BAD_REQUEST,
                Json(ErrorResponse::new(
                    ErrorCode::InvalidUsername,
                    &self.to_string(),
                    None,
                )),
            )
                .into_response(),
            RegistrationError::UserIdTaken => (
                StatusCode::BAD_REQUEST,
                Json(ErrorResponse::new(
                    ErrorCode::UserInUse,
                    &self.to_string(),
                    None,
                )),
            )
                .into_response(),
        }
    }
}
